"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _vue = require("vue");
var _xeUtils = _interopRequireDefault(require("xe-utils"));
var _core = require("@vxe-ui/core");
var _vn = require("../../ui/src/vn");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
var _default = exports.default = (0, _vue.defineComponent)({
  name: 'VxeToolbar',
  props: {
    loading: Boolean,
    refresh: [Boolean, Object],
    import: [Boolean, Object],
    export: [Boolean, Object],
    print: [Boolean, Object],
    zoom: [Boolean, Object],
    custom: [Boolean, Object],
    buttons: {
      type: Array,
      default: () => (0, _core.getConfig)().toolbar.buttons
    },
    tools: {
      type: Array,
      default: () => (0, _core.getConfig)().toolbar.tools
    },
    perfect: {
      type: Boolean,
      default: () => (0, _core.getConfig)().toolbar.perfect
    },
    size: {
      type: String,
      default: () => (0, _core.getConfig)().toolbar.size || (0, _core.getConfig)().size
    },
    className: [String, Function]
  },
  emits: ['button-click', 'tool-click'],
  setup(props, context) {
    const {
      slots,
      emit
    } = context;
    const xID = _xeUtils.default.uniqueId();
    const {
      computeSize
    } = (0, _core.useSize)(props);
    const reactData = (0, _vue.reactive)({
      isRefresh: false,
      columns: []
    });
    const refElem = (0, _vue.ref)();
    const refMaps = {
      refElem
    };
    const $xeToolbar = {
      xID,
      props,
      context,
      reactData,
      getRefMaps: () => refMaps
    };
    let toolbarMethods = {};
    const $xeGrid = (0, _vue.inject)('$xeGrid', null);
    let $xeTable;
    const connectFlag = (0, _vue.ref)(0);
    const computeRefreshOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.refresh, props.refresh);
    });
    const computeImportOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.import, props.import);
    });
    const computeExportOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.export, props.export);
    });
    const computePrintOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.print, props.print);
    });
    const computeZoomOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.zoom, props.zoom);
    });
    const computeCustomOpts = (0, _vue.computed)(() => {
      return Object.assign({}, (0, _core.getConfig)().toolbar.custom, props.custom);
    });
    const computeTableCustomOpts = (0, _vue.computed)(() => {
      if (connectFlag.value || $xeTable) {
        if ($xeTable) {
          const {
            computeCustomOpts
          } = $xeTable.getComputeMaps();
          return computeCustomOpts.value;
        }
      }
      return {};
    });
    const computeTrigger = (0, _vue.computed)(() => {
      const tableCustomOpts = computeTableCustomOpts.value;
      return tableCustomOpts.trigger;
    });
    const checkTable = () => {
      if ($xeTable) {
        return true;
      }
      _core.log.err('vxe.error.barUnableLink');
    };
    const handleClickSettingEvent = ({
      $event
    }) => {
      if ($xeTable) {
        if ($xeTable.triggerCustomEvent) {
          $xeTable.triggerCustomEvent($event);
        } else {
          _core.log.err('vxe.error.reqModule', ['VxeTableCustomModule']);
        }
      }
    };
    const handleMouseenterSettingEvent = ({
      $event
    }) => {
      if ($xeTable) {
        $xeTable.customOpenEvent($event);
      } else {
        _core.log.err('vxe.error.reqModule', ['VxeTableCustomModule']);
      }
    };
    const handleMouseleaveSettingEvent = ({
      $event
    }) => {
      const {
        customStore
      } = $xeTable.reactData;
      customStore.activeBtn = false;
      setTimeout(() => {
        if (!customStore.activeBtn && !customStore.activeWrapper) {
          $xeTable.customColseEvent($event);
        }
      }, 350);
    };
    const refreshEvent = evnt => {
      const {
        isRefresh
      } = reactData;
      const refreshOpts = computeRefreshOpts.value;
      if (!isRefresh) {
        const queryMethod = refreshOpts.queryMethod || refreshOpts.query;
        if (queryMethod) {
          reactData.isRefresh = true;
          try {
            Promise.resolve(queryMethod({})).catch(e => e).then(() => {
              reactData.isRefresh = false;
            });
          } catch (e) {
            reactData.isRefresh = false;
          }
        } else if ($xeGrid) {
          reactData.isRefresh = true;
          $xeGrid.triggerToolbarCommitEvent({
            code: refreshOpts.code || 'reload'
          }, evnt).catch(e => e).then(() => {
            reactData.isRefresh = false;
          });
        }
      }
    };
    const zoomEvent = evnt => {
      if ($xeGrid) {
        $xeGrid.triggerZoomEvent(evnt);
      }
    };
    const btnEvent = (evnt, item) => {
      const {
        code
      } = item;
      if (code) {
        if ($xeGrid) {
          $xeGrid.triggerToolbarBtnEvent(item, evnt);
        } else {
          const gCommandOpts = _core.commands.get(code);
          const params = {
            code,
            button: item,
            $table: $xeTable,
            $grid: $xeGrid,
            $event: evnt
          };
          if (gCommandOpts) {
            if (gCommandOpts.commandMethod) {
              gCommandOpts.commandMethod(params);
            } else {
              if (process.env.NODE_ENV === 'development') {
                _core.log.err('vxe.error.notCommands', [code]);
              }
            }
          }
          $xeToolbar.dispatchEvent('button-click', params, evnt);
        }
      }
    };
    const tolEvent = (evnt, item) => {
      const {
        code
      } = item;
      if (code) {
        if ($xeGrid) {
          $xeGrid.triggerToolbarTolEvent(item, evnt);
        } else {
          const gCommandOpts = _core.commands.get(code);
          const params = {
            code,
            tool: item,
            $table: $xeTable,
            $grid: $xeGrid,
            $event: evnt
          };
          if (gCommandOpts) {
            if (gCommandOpts.commandMethod) {
              gCommandOpts.commandMethod(params);
            } else {
              if (process.env.NODE_ENV === 'development') {
                _core.log.err('vxe.error.notCommands', [code]);
              }
            }
          }
          $xeToolbar.dispatchEvent('tool-click', params, evnt);
        }
      }
    };
    const importEvent = () => {
      if (checkTable()) {
        $xeTable.openImport();
      }
    };
    const exportEvent = () => {
      if (checkTable()) {
        $xeTable.openExport();
      }
    };
    const printEvent = () => {
      if (checkTable()) {
        $xeTable.openPrint();
      }
    };
    const renderDropdowns = (item, isBtn) => {
      const {
        dropdowns
      } = item;
      const downVNs = [];
      if (dropdowns) {
        return dropdowns.map((child, index) => {
          if (child.visible === false) {
            return (0, _vue.createCommentVNode)();
          }
          return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
            key: index,
            disabled: child.disabled,
            loading: child.loading,
            type: child.type,
            icon: child.icon,
            circle: child.circle,
            round: child.round,
            status: child.status,
            content: child.name,
            onClick: evnt => isBtn ? btnEvent(evnt, child) : tolEvent(evnt, child)
          });
        });
      }
      return downVNs;
    };
    /**
     * 渲染按钮
     */
    const renderBtns = () => {
      const {
        buttons
      } = props;
      const buttonsSlot = slots.buttons;
      if (buttonsSlot) {
        return (0, _vn.getSlotVNs)(buttonsSlot({
          $grid: $xeGrid,
          $table: $xeTable
        }));
      }
      const btnVNs = [];
      if (buttons) {
        buttons.forEach(item => {
          const {
            dropdowns,
            buttonRender
          } = item;
          if (item.visible !== false) {
            const compConf = buttonRender ? _core.renderer.get(buttonRender.name) : null;
            if (buttonRender && compConf && compConf.renderToolbarButton) {
              const toolbarButtonClassName = compConf.toolbarButtonClassName;
              const params = {
                $grid: $xeGrid,
                $table: $xeTable,
                button: item
              };
              btnVNs.push((0, _vue.h)('span', {
                class: ['vxe-button--item', toolbarButtonClassName ? _xeUtils.default.isFunction(toolbarButtonClassName) ? toolbarButtonClassName(params) : toolbarButtonClassName : '']
              }, (0, _vn.getSlotVNs)(compConf.renderToolbarButton(buttonRender, params))));
            } else {
              btnVNs.push((0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
                disabled: item.disabled,
                loading: item.loading,
                type: item.type,
                icon: item.icon,
                circle: item.circle,
                round: item.round,
                status: item.status,
                content: item.name,
                destroyOnClose: item.destroyOnClose,
                placement: item.placement,
                transfer: item.transfer,
                onClick: evnt => btnEvent(evnt, item)
              }, dropdowns && dropdowns.length ? {
                dropdowns: () => renderDropdowns(item, true)
              } : {}));
            }
          }
        });
      }
      return btnVNs;
    };
    /**
     * 渲染右侧工具
     */
    const renderRightTools = () => {
      const {
        tools
      } = props;
      const toolsSlot = slots.tools;
      if (toolsSlot) {
        return (0, _vn.getSlotVNs)(toolsSlot({
          $grid: $xeGrid,
          $table: $xeTable
        }));
      }
      const btnVNs = [];
      if (tools) {
        tools.forEach((item, tIndex) => {
          const {
            dropdowns,
            toolRender
          } = item;
          if (item.visible !== false) {
            const rdName = toolRender ? toolRender.name : null;
            const compConf = toolRender ? _core.renderer.get(rdName) : null;
            if (toolRender && compConf && compConf.renderToolbarTool) {
              const toolbarToolClassName = compConf.toolbarToolClassName;
              const params = {
                $grid: $xeGrid,
                $table: $xeTable,
                tool: item
              };
              btnVNs.push((0, _vue.h)('span', {
                key: rdName,
                class: ['vxe-tool--item', toolbarToolClassName ? _xeUtils.default.isFunction(toolbarToolClassName) ? toolbarToolClassName(params) : toolbarToolClassName : '']
              }, (0, _vn.getSlotVNs)(compConf.renderToolbarTool(toolRender, params))));
            } else {
              btnVNs.push((0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
                key: tIndex,
                disabled: item.disabled,
                loading: item.loading,
                type: item.type,
                icon: item.icon,
                circle: item.circle,
                round: item.round,
                status: item.status,
                content: item.name,
                destroyOnClose: item.destroyOnClose,
                placement: item.placement,
                transfer: item.transfer,
                onClick: evnt => tolEvent(evnt, item)
              }, dropdowns && dropdowns.length ? {
                dropdowns: () => renderDropdowns(item, false)
              } : {}));
            }
          }
        });
      }
      return btnVNs;
    };
    const renderToolImport = () => {
      const importOpts = computeImportOpts.value;
      return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
        key: 'import',
        circle: true,
        icon: importOpts.icon || (0, _core.getIcon)().TOOLBAR_TOOLS_IMPORT,
        title: (0, _core.getI18n)('vxe.toolbar.import'),
        onClick: importEvent
      });
    };
    const renderToolExport = () => {
      const exportOpts = computeExportOpts.value;
      return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
        key: 'export',
        circle: true,
        icon: exportOpts.icon || (0, _core.getIcon)().TOOLBAR_TOOLS_EXPORT,
        title: (0, _core.getI18n)('vxe.toolbar.export'),
        onClick: exportEvent
      });
    };
    const renderToolPrint = () => {
      const printOpts = computePrintOpts.value;
      return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
        key: 'print',
        circle: true,
        icon: printOpts.icon || (0, _core.getIcon)().TOOLBAR_TOOLS_PRINT,
        title: (0, _core.getI18n)('vxe.toolbar.print'),
        onClick: printEvent
      });
    };
    const renderToolRefresh = () => {
      const refreshOpts = computeRefreshOpts.value;
      return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
        key: 'refresh',
        circle: true,
        icon: reactData.isRefresh ? refreshOpts.iconLoading || (0, _core.getIcon)().TOOLBAR_TOOLS_REFRESH_LOADING : refreshOpts.icon || (0, _core.getIcon)().TOOLBAR_TOOLS_REFRESH,
        title: (0, _core.getI18n)('vxe.toolbar.refresh'),
        onClick: refreshEvent
      });
    };
    const renderToolZoom = () => {
      const zoomOpts = computeZoomOpts.value;
      return $xeGrid ? (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), {
        key: 'zoom',
        circle: true,
        icon: $xeGrid.isMaximized() ? zoomOpts.iconOut || (0, _core.getIcon)().TOOLBAR_TOOLS_MINIMIZE : zoomOpts.iconIn || (0, _core.getIcon)().TOOLBAR_TOOLS_FULLSCREEN,
        title: (0, _core.getI18n)(`vxe.toolbar.zoom${$xeGrid.isMaximized() ? 'Out' : 'In'}`),
        onClick: zoomEvent
      }) : (0, _vue.createCommentVNode)();
    };
    const renderToolCustom = () => {
      const customOpts = computeCustomOpts.value;
      const btnTrigger = computeTrigger.value;
      const customBtnOns = {};
      if (btnTrigger === 'manual') {
        // 手动触发
      } else if (btnTrigger === 'hover') {
        // hover 触发
        customBtnOns.onMouseenter = handleMouseenterSettingEvent;
        customBtnOns.onMouseleave = handleMouseleaveSettingEvent;
      } else {
        // 点击触发
        customBtnOns.onClick = handleClickSettingEvent;
      }
      return (0, _vue.h)((0, _vue.resolveComponent)('vxe-button'), Object.assign({
        key: 'custom',
        circle: true,
        icon: customOpts.icon || (0, _core.getIcon)().TOOLBAR_TOOLS_CUSTOM,
        title: (0, _core.getI18n)('vxe.toolbar.custom'),
        className: 'vxe-toolbar-custom-target'
      }, customBtnOns));
    };
    toolbarMethods = {
      dispatchEvent(type, params, evnt) {
        emit(type, (0, _core.createEvent)(evnt, {
          $toolbar: $xeToolbar
        }, params));
      },
      syncUpdate(params) {
        const {
          collectColumn
        } = params;
        $xeTable = params.$table;
        reactData.columns = collectColumn;
        connectFlag.value++;
      }
    };
    Object.assign($xeToolbar, toolbarMethods);
    (0, _vue.nextTick)(() => {
      const {
        refresh
      } = props;
      const refreshOpts = computeRefreshOpts.value;
      const queryMethod = refreshOpts.queryMethod || refreshOpts.query;
      if (refresh && !$xeGrid && !queryMethod) {
        _core.log.warn('vxe.error.notFunc', ['queryMethod']);
      }
      const customOpts = computeCustomOpts.value;
      if (process.env.NODE_ENV === 'development') {
        if (customOpts.isFooter) {
          _core.log.warn('vxe.error.delProp', ['toolbar.custom.isFooter', 'table.custom-config.showFooter']);
        }
        if (customOpts.showFooter) {
          _core.log.warn('vxe.error.delProp', ['toolbar.custom.showFooter', 'table.custom-config.showFooter']);
        }
        if (customOpts.immediate) {
          _core.log.warn('vxe.error.delProp', ['toolbar.custom.immediate', 'table.custom-config.immediate']);
        }
        if (customOpts.trigger) {
          _core.log.warn('vxe.error.delProp', ['toolbar.custom.trigger', 'table.custom-config.trigger']);
        }
      }
    });
    const renderVN = () => {
      const {
        perfect,
        loading,
        refresh,
        zoom,
        custom,
        className
      } = props;
      const vSize = computeSize.value;
      return (0, _vue.h)('div', {
        ref: refElem,
        class: ['vxe-toolbar', className ? _xeUtils.default.isFunction(className) ? className({
          $toolbar: $xeToolbar
        }) : className : '', {
          [`size--${vSize}`]: vSize,
          'is--perfect': perfect,
          'is--loading': loading
        }]
      }, [(0, _vue.h)('div', {
        class: 'vxe-buttons--wrapper'
      }, renderBtns()), (0, _vue.h)('div', {
        class: 'vxe-tools--wrapper'
      }, renderRightTools()), (0, _vue.h)('div', {
        class: 'vxe-tools--operate'
      }, [props.import ? renderToolImport() : (0, _vue.createCommentVNode)(), props.export ? renderToolExport() : (0, _vue.createCommentVNode)(), props.print ? renderToolPrint() : (0, _vue.createCommentVNode)(), refresh ? renderToolRefresh() : (0, _vue.createCommentVNode)(), zoom && $xeGrid ? renderToolZoom() : (0, _vue.createCommentVNode)(), custom ? renderToolCustom() : (0, _vue.createCommentVNode)()])]);
    };
    $xeToolbar.renderVN = renderVN;
    return $xeToolbar;
  },
  render() {
    return this.renderVN();
  }
});